#! /usr/bin/Rscript

suppressPackageStartupMessages({
    library(dplyr)
    library(reshape2)

    library(cleangeo)
    library(rworldmap)
    library(rgdal)

    library(scales)
    library(ggplot2)
    library(ggalt)
    library(ggthemes)
    library(cowplot)
})

world.map <- getMap("coarse")
ISO.A2 <- data.frame(cc=levels(world.map$ISO_A2), stringsAsFactors=FALSE)
world.gg <- fortify(clgeo_Clean(world.map), region="ISO_A2")

lseq = seq(-90, 90, by=.25)
boundary <- data.frame(
    long = c(rep(-180, length(lseq)), rep(180, length(lseq))),
    lat  = c(lseq,                    rev(lseq)))

lmks <- read.csv("../figure/landmarks-phase-2.csv")
svg("atlas.svg", width=7, height=4.33, family="Linux Biolinum")
print(
ggplot() +
    geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
    geom_map(data=world.gg, map=world.gg, aes(map_id=id),
             color="#cccccc", fill="#eeeeee", size=0.1) +
    geom_point(data=lmks, aes(x=longitude, y=latitude, colour=type),
               size=0.25, alpha=0.75) +
    scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
    scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
    scale_colour_brewer(palette="Set1",
                        labels=c("Anchor", "Probe")) +
    coord_proj("+proj=wintri") +
    theme_map(base_size=10) +
    theme(
        panel.background=element_blank(),
        panel.border=element_blank(),
        panel.spacing=unit(0.66666, "in"),
        plot.background=element_blank(),
        strip.background=element_blank(),
        strip.text=element_blank(),
        plot.margin=unit(c(0,0,0,0), "lines"),
        legend.title=element_blank()
        )
)
invisible(dev.off())

crwd <- read.csv("../figure/crowdsourced-hosts.csv")
svg("crowd.svg", width=7, height=4.33, family="Linux Biolinum")
print(
ggplot() +
    geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
    geom_map(data=world.gg, map=world.gg, aes(map_id=id),
             color="#cccccc", fill="#eeeeee", size=0.1) +
    geom_point(data=crwd, aes(x=client_lon, y=client_lat),
               size=0.25, alpha=0.75) +
    scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
    scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
    coord_proj("+proj=wintri") +
    theme_map(base_size=10) +
    theme(
        panel.background=element_blank(),
        panel.border=element_blank(),
        panel.spacing=unit(0.66666, "in"),
        plot.background=element_blank(),
        strip.background=element_blank(),
        strip.text=element_blank(),
        plot.margin=unit(c(0,0,0,0), "lines"),
        legend.title=element_blank()
        )
)
invisible(dev.off())

perf <- read.csv("perf.csv")
perf.f <- subset(perf, phase=="first")
perf.s <- subset(perf, phase=="second")
svg("perf.demo.svg", width=7, height=9)
p1 <- ggplot() +
    geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
    geom_map(data=world.gg, map=world.gg, aes(map_id=id),
             color="#cccccc", fill="#eeeeee", size=0.1) +
    geom_point(data=perf.f, aes(x=lon, y=lat), size=0.5) +
    scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
    scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
    coord_proj("+proj=wintri") +
    theme_map(base_size=10) +
    theme(
        panel.background=element_blank(),
        panel.border=element_blank(),
        panel.spacing=unit(0.66666, "in"),
        plot.background=element_blank(),
        strip.background=element_blank(),
        strip.text=element_blank(),
        plot.margin=margin(b=0.5, unit="cm"),
        legend.title=element_blank()
        )
p2 <- ggplot() +
    geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
    geom_map(data=world.gg, map=world.gg, aes(map_id=id),
             color="#cccccc", fill="#eeeeee", size=0.1) +
    geom_point(data=perf.s, aes(x=lon, y=lat),
               size=0.5) +
    scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
    scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
    coord_proj("+proj=wintri", xlim=c(-15, 35), ylim=c(35, 72.5)) +
    theme_map(base_size=10) +
    theme(
        panel.background=element_blank(),
        panel.border=element_blank(),
        panel.spacing=unit(0.66666, "in"),
        plot.background=element_blank(),
        strip.background=element_blank(),
        strip.text=element_blank(),
        plot.margin=margin(t=0.5, unit="cm"),
        legend.title=element_blank()
        )
plot_grid(p1, p2, ncol=1)
invisible(dev.off())

draw.compare.claims <- function(pname, pcode) {
    claims <- subset(read.csv("../figure/proxy-credibility-dc-cont.csv"),
                     provider==pname,
                     c("alleged.cc", "probable.cc"))
    claims$alleged.cc <- toupper(as.character(claims$alleged.cc))
    claims$probable.cc <- toupper(as.character(claims$probable.cc))

    ccount <- (left_join(ISO.A2, claims,
                         by=c("cc"="alleged.cc")) %>%
               group_by(cc) %>%
               summarise(n.claimed=sum(!is.na(probable.cc))))

    pcount <- (left_join(ISO.A2, claims,
                         by=c("cc"="probable.cc")) %>%
               group_by(cc) %>%
               summarise(n.probable=sum(!is.na(alleged.cc))))

    xcount <- (inner_join(ccount, pcount, by='cc') %>%
               mutate(b.claimed = ordered(n.claimed),
                      b.probable = ordered(n.probable)))

    # These color scales are essentially what scale_fill_distiller would
    # produce, except that we use the neutral #eeeeee for zero.
    ccolors <- c("#EEEEEE", gradient_n_pal(brewer_pal(palette="YlGn")(5))
                 (seq(0, 1, length.out=length(levels(xcount$b.claimed)) - 1)))
    pcolors <- c("#EEEEEE", gradient_n_pal(brewer_pal(palette="YlGn")(5))
                 (seq(0, 1, length.out=length(levels(xcount$b.probable)) - 1)))

    claims.map <- inner_join(world.gg, xcount, by=c('id'='cc'))

    # open the device before doing anything that might involve measuring text
    svg(sprintf("cvp-%s.svg", pcode), width=14.520833, height=4.319291,
        family="Linux Biolinum")
    c.map <- ggplot() +
        geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
        geom_map(data=claims.map, map=claims.map,
                 aes(map_id=id, fill=b.claimed),
                 color="#cccccc", size=0.1) +
        scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
        scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
        scale_fill_manual(values=ccolors) +
        coord_proj("+proj=wintri") +
        theme_map(base_size=10) +
        theme(
            plot.background=element_blank(),
            plot.margin=margin(r=0.5, unit="cm"),
            legend.title=element_blank(),
            legend.key=element_blank(),
            legend.key.height=unit(0.9,"lines"),
            legend.text=element_text(size=rel(0.8), margin=margin(l=2)),
            legend.position=c(0,0),
            legend.justification=c(0,0)
        )

    p.map <- ggplot() +
        geom_polygon(data=boundary, aes(x=long, y=lat), fill="#f7fcfd") +
        geom_map(data=claims.map, map=claims.map,
                 aes(map_id=id, fill=b.probable),
                 color="#cccccc", size=0.1) +
        scale_x_continuous(limits=c(-180,180), expand=c(0,0)) +
        scale_y_continuous(limits=c(-90,90), expand=c(0,0)) +
        scale_fill_manual(values=pcolors) +
        coord_proj("+proj=wintri") +
        theme_map(base_size=10) +
        theme(
            plot.background=element_blank(),
            plot.margin=margin(l=0.5, unit="cm"),
            legend.title=element_blank(),
            legend.key=element_blank(),
            legend.key.height=unit(0.9,"lines"),
            legend.text=element_text(size=rel(0.8), margin=margin(l=2)),
            legend.position=c(0,0),
            legend.justification=c(0,0)
        )

    print(plot_grid(c.map, p.map))
    dev.off()
    invisible(NULL)
}

draw.compare.claims("hma", "A")
draw.compare.claims("pure", "B")
draw.compare.claims("express", "C")
draw.compare.claims("ipvanish", "D")
draw.compare.claims("ib", "E")
draw.compare.claims("tbear", "F")
draw.compare.claims("total", "G")

